/*
 * utils.c
 *
 * Some useful utils to use with programs.
 */

#include "utils.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

matrix 
read_matrix(const char *file, int *s, int *d) 
{
  int i, j, value;
  FILE *fp;
  matrix B;

  if (file[0] == '-' && file[1] == '\0') fp = stdin;
  else {
    if ((fp=fopen(file, "r")) == NULL) {
      perror("Error when opening the set file");
      return NULL;
    }
    if (setvbuf(fp, NULL, _IOFBF, BUFSIZ)) {
      perror("Error when creating buffer for reading the set");
      return NULL;
    }
  }

  if (fscanf(fp, "%i %i", s, d) == EOF) {
    perror("Error while reading the set file");
    if (fp!=stdin) fclose(fp);
    return NULL;
  }

  if ((B = malloc((*s) * sizeof(vector))) == NULL) {
    perror("Error while allocating space for set");
    if (fp!=stdin) fclose(fp);
    return NULL;
  }
  for (i=0; i<(*s); i++) {
    if ((B[i] = malloc((*d) * sizeof(char))) == NULL) {
      perror("Error while allocating space for set");
      if (fp!=stdin) fclose(fp);
      free_matrix(B, i);
      return NULL;
    }
  }

  for (i=0; i<(*s); i++) {
    for (j=0; j<(*d); j++) {
      if (fscanf(fp, "%i", &value) == EOF) {
	perror("Error while reading set's vectors");
	if (fp != stdin) fclose(fp);
	free_matrix(B, *s);
	return NULL;
      }
      B[i][j] = (char)value;
    }
  }

  return B;
}


matrix
read_sparse_matrix(const char *file, int *s, int *d)
{
  int i, value, ones, j;
  FILE *fp;
  matrix B;


  if (file[0] == '-' && file[1] == '\0') fp = stdin;
  else {
    if ((fp=fopen(file, "r")) == NULL) {
      perror("Error when opening the set file");
      return NULL;
    }
    if (setvbuf(fp, NULL, _IOFBF, BUFSIZ)) {
      perror("Error when creating buffer for reading the set");
      return NULL;
    }
  }

  if (fscanf(fp, "%i %i", s, d) == EOF) {
    perror("Error while reading the set file");
    if (fp!=stdin) fclose(fp);
    return NULL;
  }

  if ((B = malloc((*s) * sizeof(vector))) == NULL) {
    perror("Error while allocating space for set");
    if (fp!=stdin) fclose(fp);
    return NULL;
  }
  for (i=0; i<(*s); i++) {
    if ((B[i] = malloc((*d) * sizeof(char))) == NULL) {
      perror("Error while allocating space for set");
      if (fp!=stdin) fclose(fp);
      free_matrix(B, i);
      return NULL;
    }
    memset(B[i], 0, *d);
  }


  /* Read the matrix; at first there should be an integer representing
   * how many 1s there are in this row.
   */
  for (i = 0; i < *s; i++) {
    if (fscanf(fp, "%i", &ones) == EOF) {
      perror("Error when reading data matrix");
      return NULL;
    }
    for (j = 0; j < ones; j++) {
      if (fscanf(fp, "%i", &value) == EOF) {
	perror("Error when reading data matrix");
	return NULL;
      }
      if (value < 1) {
	fprintf(stderr, "Error when reading data: value %i (row %i, %i:th value) "
		"lesser than 1.\n", value, i, j);
	return NULL;
      } else if (value > *d) {
	fprintf(stderr, "Error when reading data: value %i (row %i, %i:th value) "
		"greater than number of columns (%i)\n", value, i, j, *d);
	return NULL;
      }
      B[i][value-1] = 1;
    }
  }

  return B;
}


int 
print_matrix(const char *file, matrix S, int s, int d) 
{
  int i, j;
  FILE *fp;

  if (file[0] == '-' && file[1] == '\0') fp = stdout;
  else {
    fp = fopen(file, "w");
    if (fp == NULL) {
      perror("Error when opening file for writing");
      return 0;
    }
    if (setvbuf(fp, NULL, _IOFBF, BUFSIZ)) {
      perror("Error when creating buffer for writing");
      fclose(fp);
      return 0;
    }
  }

  fprintf(fp, "%i\n%i\n", s, d);
  for (i=0; i<s; i++) {
    for (j=0; j<d; j++) fprintf(fp, "%i ", (int)S[i][j]);
    putc('\n', fp);
  }

  if (fp != stdout)
    fclose(fp);
  return 1;
}

int
print_sparse_matrix(const char *file, matrix S, int s, int d)
{
  int i, j, ones;
  FILE *fp;
  if (file[0] == '-' && file[1] == '\0') fp = stdout;
  else {
    fp = fopen(file, "w");
    if (fp == NULL) {
      perror("Error when opening file for writing");
      return 0;
    }
    if (setvbuf(fp, NULL, _IOFBF, BUFSIZ)) {
      perror("Error when creating buffer for writing");
      fclose(fp);
      return 0;
    }
  }

  fprintf(fp, "%i\n%i\n", s, d);
  for (i=0; i<s; i++) {
    /* compute ones */
    for (j=0, ones=0; j<d; j++) ones+=S[i][j];
    /* print ones */
    fprintf(fp, "%i ", ones);
    for (j=0; j<d; j++) {
      if (S[i][j]) fprintf(fp, "%i ", j+1);
    }
    putc('\n', fp);
  }

  if (fp != stdout)
    fclose(fp);
  return 1;
}


void 
free_matrix(matrix M, int n) 
{
  int i;

  for (i=0; i<n; i++) free(M[i]);
  free(M);
}


FILE *
init_seed(unsigned int seed)
{
  FILE *fp = NULL;

  if (seed != 0) {
    srand(seed);
  } else {
    fp = fopen("/dev/urandom", "r");
    if (fp == NULL) {
      perror("virhe avattaessa satunnaislukulaitetta");
      exit(1);
    }
  }
  return fp;
}


unsigned int
give_rand(FILE *randdev)
{
  unsigned int random;
  int i;

  if (randdev == NULL) {
    return (unsigned)rand();
  } else {
    i = fread(&random, sizeof(random), 1, randdev);
    if (i == 0) {
      if (feof(randdev)) {
	fprintf(stderr, "EOF luettaessa satunnaislukulaitetta\n");
	fclose(randdev);
	exit(1);
      } else if (ferror(randdev)) {
	perror("virhe luettaessa satunnaislukulaitetta");
	fclose(randdev);
	exit(1);
      }
    }
    return random;
  }
}

  
